import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Scanner;

public class Ospedale {
	private String nomeOspedale;
	
	private ArrayList<Medico> listaMedici;
	private LinkedList<Paziente> listaPazienti;
	
	
	public Ospedale(String N)
	{
		nomeOspedale = N;
		listaMedici = new ArrayList<Medico>();
		listaPazienti = new LinkedList<Paziente>();
	}
	
	public void initMediciFromFile(String path)
	{
		File f = new File(path);
		Scanner input = null;
		
		String tempString = new String();
		Medico tempMedico;
		
		try {
			input = new Scanner(f);
			while (input.hasNextLine())
			{
				tempString = input.nextLine();
				tempMedico = new Medico(tempString.split("\t")[0], tempString.split("\t")[1], 
						tempString.split("\t")[2], tempString.split("\t")[3], Integer.parseInt(tempString.split("\t")[4]));
				listaMedici.add(tempMedico);
			}
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.out.println("Errore");
		}
		finally
		{
			if (input != null)
				input.close();
		}
	}
	
	public void initPazientiFromFile(String path)
	{
		File f = new File(path);
		Scanner input = null;
		
		String tempString = new String();
		Paziente tempPaziente;
		
		try {
			input = new Scanner(f);
			while (input.hasNextLine())
			{
				tempString = input.nextLine();
				tempPaziente = new Paziente(tempString.split("\t")[0], tempString.split("\t")[1], 
						tempString.split("\t")[2], tempString.split("\t")[3]);
				listaPazienti.add(tempPaziente);
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (Exception e)
		{
			System.out.println("Errore");
		}
		finally
		{
			if (input != null)
				input.close();
		}
	}
	
	public String getNome()
	{
		return nomeOspedale;
	}
	
	public void aggiungiMedici(Medico M)
	{
		listaMedici.add(M);
		System.out.println("Medico Aggiunto");
	}
	
	public void aggiungiMedici(ArrayList<Medico> LM)
	{
		listaMedici.addAll(LM);
		System.out.println("Medici Aggiunti");
	}
	
	public void aggiungiPazienti(Paziente P)
	{
		listaPazienti.add(P);
		System.out.println("Paziente Aggiunto");
	}
	
	public void aggiungiPazienti(ArrayList<Paziente> LP)
	{
		listaPazienti.addAll(LP);
		System.out.println("Pazienti Aggiunti");
	}
	
	public void stampaMedici()
	{
		System.out.println("Lista Medici:");
		for (int i = 0; i < listaMedici.size(); i++)
		{
			System.out.println((i+1) + ": " + listaMedici.get(i).getInfo());
		}
	}
	
	public void stampaPazienti()
	{
		System.out.println("Lista Pazienti:");
		for (int i = 0; i < listaPazienti.size(); i++)
		{
			System.out.println((i+1) + ": " + listaPazienti.get(i).getInfo());
		}
	}
	
	public void assegnaPrenotazioneMedico(Prenotazione P)
	{
		int idx = trovaMedicoLibero();
		if (idx > -1)
			listaMedici.get(idx).accettaPazienteInCoda(P);
		else
			System.out.println("Nessun medico disponibile ad accettare la prenotazione " + P.stampaInfoPrenotazione());
	}
	
	private int trovaMedicoLibero () 
	{
		// Restituisce l'indice della lista dei medici con il medico con meno utenti in coda
		int idx_min = -1;
		if (listaMedici.size() != 0)
		{
			// Devo trovare il primo medico disponibile
			for(int i = 0; i < listaMedici.size(); i++)
			{
				if(listaMedici.get(i).isAvailable())
				{
					idx_min = i;
					break;
				}
			}
		}
		if (idx_min > -1)
		{
			for (int i = idx_min + 1; i < listaMedici.size(); i++) 
			{
				if (listaMedici.get(i).isAvailable() &&  (listaMedici.get(i).utentiInCoda() < listaMedici.get(idx_min).utentiInCoda()))
				{
					idx_min = i;
				}
			}
			
		}
		return idx_min;
	}
	
	public void stampaStatoMedici()
	{
		for (int i = 0; i < listaMedici.size(); i++)
		{
			System.out.println("Medico: " + listaMedici.get(i).getCognome());
			listaMedici.get(i).stampaStatoCoda();
		}
	}
	
	
	public void curaPazienti() {
		for (int i = 0; i < listaMedici.size(); i++)
			listaMedici.get(i).curaProssimoPaziente(this);
	}
	
}
